#ifndef MongoDB_H
#define MongoDB_H
#ifdef MONGO_DRIVERS

#include "MongoConsts.h"
#include "RootConfig.h"
#include "MongoAccess.h"

#include <QDateTime>
#include <QDebug>
#include <QMetaObject>
#include <QObject>
#include <QSettings>
#include <QString>
#include <QTime>

#include <cstdlib>
#include <memory>

#ifndef Q_MOC_RUN
#include <bsoncxx/stdx/make_unique.hpp>
#include <bsoncxx/stdx/optional.hpp>
#include <bsoncxx/stdx/string_view.hpp>

#include <mongocxx/instance.hpp>
#include <mongocxx/client.hpp>
#include <mongocxx/logger.hpp>
#include <mongocxx/pool.hpp>
#include <mongocxx/uri.hpp>
#endif

using namespace std;

struct stack_element
{
    bsoncxx::document::view doc;
    RootConfig my_root;
};

class MongoDB : public QObject
{
    Q_OBJECT
public:
    MongoDB();
    virtual ~MongoDB();
    void mongo_init();
    bool try_connection();

    RootConfig read_config(QString program_type,
                           QString program_index = PROGRAM_INDEX_DEFAULT,
                           QString config_name = PROGRAM_CONFIG_DEFAULT);
    bool write_config(const RootConfig &root);
    RootConfig read_history_config(QString program_type,
                                   QString run_index = PROGRAM_INDEX_DEFAULT,
                                   int run_number = 0);
    bool write_history_config(RootConfig root);
    int check_run_number(QString program_type, QString run_index, int run_number);
    QList<RootConfig> read_all_config(QString collection_name);
    bool write_device_config(RootConfig root);
    QMap<QString, QStringList> get_config_map(QString program_type);
    QMap<QString, QMap<QString, RootConfig>> get_rootconfig_map(QString program_type);
    QStringList get_types_list(QString collection_name);
    QDateTime get_date_of_modification(QString program_type,
                                       QString program_index = PROGRAM_INDEX_DEFAULT,
                                       QString config_name = PROGRAM_CONFIG_DEFAULT);
    bool static delete_config(QString program_type,
                              QString program_index=PROGRAM_INDEX_DEFAULT,
                              QString config_name = PROGRAM_CONFIG_DEFAULT);

    //---type cast
    static QVariant bsoncxx_to_qvariant(bsoncxx::types::value value);
    static bsoncxx::types::value qvariant_to_bsoncxx(QVariant qvar);

    //---main stream moduls
    bool root_to_mongo(QString collection_name, RootConfig root);
    void rename_date_of_creation();
    QString uri;
    QString db;

    static RootConfig doc_element_to_root(bsoncxx::document::view doc);
    builder::basic::document root_to_doc_element(const RootConfig &root);
private:
    //---preparation moduls
    bool remove_broken_docs(QString collection_name, QStringList fields_to_check);
    bool remove_duplicates(QString collection_name);
    bool remove_duplicates_from_devices(QString collection_name);

    QMap<QUuid, QString> DFS_path_map(RootConfig root);

    //---write device config
    RootConfig replace_current_to_log(RootConfig target_root);
    QString quint64_to_serial_str(quint64 value);
    quint64 serial_str_to_quint64(QString str);

    //---debug modules
    bool write_settings_to_local_saved_debug(QString type, QString name);
    void root_debug_print(RootConfig root);

    bool mongo_alive;
    bool type_cast_warning;
};
#endif
#endif // MongoDB_H

//--- garbage ---

